==> package-manager-1.99.1.tar.gz: package-manager-1.99.1/setup/parse-request.c <==
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <badpenguin-ansi.h>
#include <badpenguin-errors.h>
#include <badpenguin-chain.h>
#include <badpenguin-fs.h>

#include "parse-info.h"
#include "parse-options.h"
#include "request-version.h"
#include "setup-quit.h"


struct stRequest {
    char *reqName;
    char *reqMin;
    char *reqMax;    
    char *libName;
    char *libMin;
    char *libMax;
    int  status;
};

typedef struct stRequest TRequest;


int debug_request_data(TRequest* data) {
  printf(
    "%s,%s,%s,%s,%s,%s", 
    data->reqName,
    data->reqMin,
    data->reqMax,
    data->libName,
    data->libMin,
    data->libMax
  );
  return 0;
}


#define MAX_SEARCHDIR 5


int check_request( char *reqName, char *reqMin, char *reqMax, char *libName, char *libMin, char *libMax, int verbose ) {
  // request checking
  char tempstr[255], tempDir[255];
  DIR *lpdir;
  struct dirent *lpstdirent;
  // lib checking
  int startsearch=0, endsearch=0, lenname=0, i;
  char *searchdir[MAX_SEARCHDIR] = { "/usr/lib/", "/usr/X11R6/lib/", "/usr/local/lib/", "/lib/", "" };
  char *searchname;

  // check syntax
  if ( reqName[0] == 0) {
    printWARNING("! formato dipendenza non valido: " );
    normal_color();
    return 1;
  }

  // check if requested package is already installed
  lpdir = opendir( gostOptions.spoolroot );
  if (lpdir==NULL) {
    printERROR("! impossibile aprire %s\n", gostOptions.spoolroot);
    normal_color();
    setup_quit(ERROR_IO);
  }
  lpstdirent = readdir(lpdir);
  while (lpstdirent!=NULL) {
    if (
      ( 0 == strncmp(lpstdirent->d_name, reqName, strlen(reqName) )) &&
      ( 0 == is_directory(gostOptions.spoolroot,lpstdirent->d_name) )
    ) {
      if ( reqMin != NULL ) {
        // Confronto del minor number
        sprintf( tempstr, "%s.%s", reqName, reqMin );
        if ( request_version( lpstdirent->d_name, tempstr ) >= 0 ) {
          if (gostOptions.verbose>0) printf("- request %s resolved by %s \n", tempstr, lpstdirent->d_name );
          return 4;
        }
      } else {
        if (gostOptions.verbose>0) printf("- request %s resolved by %s \n", reqName, lpstdirent->d_name );
        return 4;
      }
    }
    lpstdirent = readdir(lpdir);
  }
  closedir(lpdir);

  // if requested package is not installed try to see if the lib exists
  if ( libName != NULL ) {
    // Controllo assoluto o relativo
    if ( libName[0] == '/' ) {      
      startsearch=MAX_SEARCHDIR-1;
      endsearch=MAX_SEARCHDIR;
      searchdir[MAX_SEARCHDIR-1] = dirpath(libName);
      searchname = dirbase(libName);
    } else {
      startsearch=0;
      endsearch=MAX_SEARCHDIR-1;
      searchdir[MAX_SEARCHDIR-1] = NULL;
      searchname=libName;
    }
    lenname = strlen(searchname);
    // controllo directories
    for (i=startsearch; i<endsearch; i++) {
      // printf("- controllo nr. %i in %s \n", i, searchdir[i] );
      strcpy(tempDir, gostOptions.targetdir);
      strcat(tempDir, searchdir[i] );
      lpdir = opendir(tempDir);
      if (lpdir!=NULL) {
        lpstdirent = readdir(lpdir);
        while (lpstdirent!=NULL) {
          if (
            ( 0 == strncmp(lpstdirent->d_name, searchname, lenname )) &&
            ( 0 == is_regularfile(tempDir,lpstdirent->d_name) )
	  ) {
          // printf("- matching: %s\n", lpstdirent->d_name );
            if (libMin != NULL) {
	      // confronto del minor number
	      // printf("- controllo minor number: %s\n", lpstdirent->d_name );
	      sprintf( tempstr, "%s.%s", libName, libMin );
  	      if ( request_version( lpstdirent->d_name, tempstr ) >= 0 ) {
    	        if (gostOptions.verbose>0) printf("- request of %s resolved by %s\n", tempstr, lpstdirent->d_name );
	        return 3;
              }
            } else {
              if (gostOptions.verbose>0) printf("- request of %s resolved by %s%s%s \n", searchname, gostOptions.targetdir, searchdir[i], lpstdirent->d_name);
  	      return 3;
            }
          }
          lpstdirent = readdir(lpdir);
        }
      }
      closedir(lpdir);
    }
  }
  // Not resolved
  if (verbose>0) {
    printWARNING("! request %s not resolved\n", reqName);
    normal_color();
  }    
  return 2;
}      



int parse_request(){
  // request chain
  TChain     *reqChain;
  TChainItem *reqItem;
  TRequest   *reqData;
  char       *p, *p2;
  // request
  int         retval;
  char        cmdline[256];
  
  // *** go ***
  if (gostOptions.verbose>0) printf("- checking dependencies \n");    
  
  // Parsing dependencies
  // p = gostPackageInfo.request;
  // DEBUG   p = "perl:::/usr/bin/perl  bash  libgtk_1:2:3:libgtk-1.2::  cdrecord:::/usr/bin/cdrecord";
  reqChain = chain_create();
  p = strdup(gostPackageInfo.request);
  p2 = strtoken( &p, " " );
  while (p2 != NULL) {
    // Add item
    reqItem  = chain_append(reqChain);
    reqData  = calloc(1,sizeof(TRequest));
    reqItem->data = reqData;
    // Decode fields  
    reqData->reqName = strtoken( &p2, ":" );
    reqData->reqMin  = strtoken( &p2, ":" );
    reqData->reqMax  = strtoken( &p2, ":" );    
    reqData->libName = strtoken( &p2, ":" );
    reqData->libMin  = strtoken( &p2, ":" );
    reqData->libMax  = strtoken( &p2, ":" );    
    // next
    p2 = strtoken( &p, " " );
  }

  // print out the number of specified dependencies  
  if (gostOptions.verbose>1) printf("- dependencies needed: %i\n", reqChain->count );
  if (reqChain->count > 0) {
  
    reqItem = reqChain->head;
    while (reqItem!=NULL) {
      reqData = reqItem->data;
      if (gostOptions.verbose>2) {
        printf(">");  
	debug_request_data(reqData);  
	printf("\n");
      }
      reqData->status = check_request(
        reqData->reqName,reqData->reqMin,reqData->reqMax,
	reqData->libName,reqData->libMin,reqData->libMax,
	1
      );
      
      // next
      reqItem=reqItem->next;
    }
  // end of if reqChain->count 
  }
  
  // Halt on any problems
  reqItem = reqChain->head;
  while (reqItem!=NULL) {
    reqData = reqItem->data;
    // try to request it
    if (reqData->status<3) {
      strcpy(cmdline, "package-setup --no-header --request ");
      strcat(cmdline, reqData->reqName);
      strcat(cmdline, ":" );
      if (reqData->reqMin  != NULL) strcat(cmdline, reqData->reqMin );
      strcat(cmdline, ":" );
      if (reqData->reqMax  != NULL) strcat(cmdline, reqData->reqMax );
      strcat(cmdline, ":" );
      if (reqData->libName != NULL) strcat(cmdline, reqData->libName );
      strcat(cmdline, ":" );
      if (reqData->libMin  != NULL) strcat(cmdline, reqData->libMin );
      strcat(cmdline, ":" );    
      if (reqData->libMax  != NULL) strcat(cmdline, reqData->libMax );
      strcat(cmdline, ":" );    
      strcat(cmdline, gostOptions.targetdir );
      // run
      if (gostOptions.verbose>0) printf("- eseguo: %s\n", cmdline);
      printf("[ installing dep %s ]----------\n", reqData->reqName );
      retval = runshell(cmdline);
      printf("----------------------------------------------------------------------\n");
      chdir(gostOptions.targetdir);
      // its ok ?
      if (retval==0) reqData->status = 5;
    }
    // check again
    if (reqData->status<3) {
      printERROR("! cannot continue if you do not solve your deps problems, first\n");
      normal_color();
      setup_quit(ERROR_FORMAT);
    }
    // next
    reqItem=reqItem->next;
  }
  
  return 0;
}

==> package-manager-1.99.4.tar.gz: package-manager-1.99.4/setup/parse-request.c <==
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <badpenguin-ansi.h>
#include <badpenguin-errors.h>
#include <badpenguin-chain.h>
#include <badpenguin-fs.h>

#include "parse-info.h"
#include "parse-options.h"
#include "request-version.h"
#include "setup-quit.h"


struct stRequest {
    char *reqName;
    char *reqMin;
    char *reqMax;    
    char *libName;
    char *libMin;
    char *libMax;
    int  status;
};

typedef struct stRequest TRequest;


int debug_request_data(TRequest* data) {
  printf(
    "%s,%s,%s,%s,%s,%s", 
    data->reqName,
    data->reqMin,
    data->reqMax,
    data->libName,
    data->libMin,
    data->libMax
  );
  return 0;
}

int push_dependencie(char *sname) {
  FILE *f = NULL;
  char fname[256];
  snprintf( fname, 256, "%s/var/spool/badpenguin/DEPENDENCIES", gostOptions.targetdir);
  f = fopen( fname, "a" );
  if (f!=NULL) {
    fprintf( f, "%s\n", sname);
  } else {
    printERROR("! impossibile aprire %s in append\n", fname);
    normal_color();
    return -1;
  }
  fclose(f);
  return 0;
}

#define MAX_SEARCHDIR 5


int check_request( char *reqName, char *reqMin, char *reqMax, char *libName, char *libMin, char *libMax, int verbose ) {
  // request checking
  char tempstr[255], tempDir[255];
  DIR *lpdir;
  struct dirent *lpstdirent;
  // lib checking
  int startsearch=0, endsearch=0, lenname=0, i;
  char *searchdir[MAX_SEARCHDIR] = { "/usr/lib/", "/usr/X11R6/lib/", "/usr/local/lib/", "/lib/", "" };
  char *searchname;
  
  // check syntax
  if ( reqName[0] == 0) {
    printWARNING("! formato dipendenza non valido: " );
    normal_color();
    return 1;
  }

  // check if requested package is already installed
  lpdir = opendir( gostOptions.spoolroot );
  if (lpdir==NULL) {
    printERROR("! impossibile aprire %s\n", gostOptions.spoolroot);
    normal_color();
    setup_quit(ERROR_IO);
  }
  lpstdirent = readdir(lpdir);
  while (lpstdirent!=NULL) {
    if (
      ( 0 == strncmp(lpstdirent->d_name, reqName, strlen(reqName) )) &&
      ( 0 == is_directory(gostOptions.spoolroot,lpstdirent->d_name) )
    ) {
      if ( reqMin != NULL ) {
        // Confronto del minor number
        sprintf( tempstr, "%s.%s", reqName, reqMin );
        if ( request_version( lpstdirent->d_name, tempstr ) >= 0 ) {
          if (gostOptions.verbose>0) printf("- request %s resolved by %s \n", tempstr, lpstdirent->d_name );
          return 4;
        }
      } else {
        if (gostOptions.verbose>0) printf("- request %s resolved by %s \n", reqName, lpstdirent->d_name );
        return 4;
      }
    }
    lpstdirent = readdir(lpdir);
  }
  closedir(lpdir);

  // if requested package is not installed try to see if the lib exists
  if ( libName != NULL ) {
    // Controllo assoluto o relativo
    if ( libName[0] == '/' ) {      
      startsearch=MAX_SEARCHDIR-1;
      endsearch=MAX_SEARCHDIR;
      searchdir[MAX_SEARCHDIR-1] = dirpath(libName);
      searchname = dirbase(libName);
    } else {
      startsearch=0;
      endsearch=MAX_SEARCHDIR-1;
      searchdir[MAX_SEARCHDIR-1] = NULL;
      searchname=libName;
    }
    lenname = strlen(searchname);
    // controllo directories
    for (i=startsearch; i<endsearch; i++) {
      // printf("- controllo nr. %i in %s \n", i, searchdir[i] );
      strcpy(tempDir, gostOptions.targetdir);
      if ( strlen(gostOptions.targetdir) > 1 ) {
        strcat( tempDir, "/" );
      }
      strcat(tempDir, searchdir[i] );
      lpdir = opendir(tempDir);
      if (lpdir!=NULL) {
        lpstdirent = readdir(lpdir);
        while (lpstdirent!=NULL) {
          if (
            ( 0 == strncmp(lpstdirent->d_name, searchname, lenname )) &&
            ( 0 == is_regularfile(tempDir,lpstdirent->d_name) )
	  ) {
          // printf("- matching: %s\n", lpstdirent->d_name );
            if (libMin != NULL) {
	      // confronto del minor number
	      // printf("- controllo minor number: %s\n", lpstdirent->d_name );
	      sprintf( tempstr, "%s.%s", libName, libMin );
  	      if ( request_version( lpstdirent->d_name, tempstr ) >= 0 ) {
    	        if (gostOptions.verbose>0) printf("- request of %s resolved by %s\n", tempstr, lpstdirent->d_name );
	        return 3;
              }
            } else {
              if (gostOptions.verbose>0) printf("- request of %s resolved by %s%s%s \n", searchname, gostOptions.targetdir, searchdir[i], lpstdirent->d_name);
  	      return 3;
            }
          }
          lpstdirent = readdir(lpdir);
        }
      }
      closedir(lpdir);
    }
  }
  // Not resolved
  if (verbose>0) {
    printWARNING("! request %s not resolved\n", reqName);
    normal_color();
  }    
  return 2;
}      



int parse_request(){
  // request chain
  TChain     *reqChain;
  TChainItem *reqItem;
  TRequest   *reqData;
  char       *p, *p2;
  // request
  int         retval;
  char        cmdline[256];
  int         runned = 0;     // Count how many dependencies has been installed
  // *** go ***
  if (gostOptions.verbose>0) printf("- checking dependencies \n");    
  
  // Parsing dependencies
  // p = gostPackageInfo.request;
  // DEBUG   p = "perl:::/usr/bin/perl  bash  libgtk_1:2:3:libgtk-1.2::  cdrecord:::/usr/bin/cdrecord";
  reqChain = chain_create();
  p = strdup(gostPackageInfo.request);
  p2 = strtoken( &p, " " );
  while (p2 != NULL) {
    // Add item
    reqItem  = chain_append(reqChain);
    reqData  = calloc(1,sizeof(TRequest));
    reqItem->data = reqData;
    // Decode fields  
    reqData->reqName = strtoken( &p2, ":" );
    reqData->reqMin  = strtoken( &p2, ":" );
    reqData->reqMax  = strtoken( &p2, ":" );    
    reqData->libName = strtoken( &p2, ":" );
    reqData->libMin  = strtoken( &p2, ":" );
    reqData->libMax  = strtoken( &p2, ":" );    
    // next
    p2 = strtoken( &p, " " );
  }

  // print out the number of specified dependencies  
  if (gostOptions.verbose>1) printf("- dependencies needed: %i\n", reqChain->count );
  if (reqChain->count > 0) {
  
    reqItem = reqChain->head;
    while (reqItem!=NULL) {
      reqData = reqItem->data;
      if (gostOptions.verbose>2) {
        printf(">");  
	debug_request_data(reqData);  
	printf("\n");
      }
      reqData->status = check_request(
        reqData->reqName,reqData->reqMin,reqData->reqMax,
	reqData->libName,reqData->libMin,reqData->libMax,
	1
      );
      
      // next
      reqItem=reqItem->next;
    }
  // end of if reqChain->count 
  }
  
  // Halt on any problems
  reqItem = reqChain->head;
  while (reqItem!=NULL) {
    reqData = reqItem->data;
    // try to request it
    if (reqData->status<3) {
      if (gostOptions.nodeps==1) {
        // we choosed to not install deps
        printWARNING("! skipping installation of %s due to --no-deps\n", reqData->reqName );
        normal_color();
	reqData->status = 6;
      } else {
        strcpy(cmdline, "package-setup --no-header --request ");
        strcat(cmdline, reqData->reqName);
        strcat(cmdline, ":" );
        if (reqData->reqMin  != NULL) strcat(cmdline, reqData->reqMin );
        strcat(cmdline, ":" );
        if (reqData->reqMax  != NULL) strcat(cmdline, reqData->reqMax );
        strcat(cmdline, ":" );
        if (reqData->libName != NULL) strcat(cmdline, reqData->libName );
        strcat(cmdline, ":" );
        if (reqData->libMin  != NULL) strcat(cmdline, reqData->libMin );
        strcat(cmdline, ":" );    
        if (reqData->libMax  != NULL) strcat(cmdline, reqData->libMax );
        strcat(cmdline, ": " );    
        strcat(cmdline, gostOptions.targetdir );
        // run
        runned++;
        if (gostOptions.verbose>0) printf("- eseguo: %s\n", cmdline);
        printf("/----- requesting %s -----\\\n", reqData->reqName );
        retval = runshell(cmdline);
        printf("\\----- requested  %s -----/\n", reqData->reqName );
        chdir(gostOptions.targetdir);
        // its ok ?
        if (retval==0) {
	  reqData->status = 5;
	} else {
	  reqData->status = 6;
        }
      }
    }
    // check again
    if (reqData->status<3) {
      printERROR("! cannot continue if you do not solve your deps problems, first\n");
      normal_color();
      setup_quit(ERROR_FORMAT);
    }
    // DEPENDENCIES
    if (reqData->status==6) {
      push_dependencie(reqData->reqName);
    }
    // next
    reqItem=reqItem->next;
  }
  
  return runned;
}
